(function() {
  var Promise, addAwsParams, buildFilters, dataTrans, debug, getKey, scanFilterFunc, _,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('lodash');

  dataTrans = require('./data-translators');

  Promise = require('bluebird');

  debug = require('debug')('dynasty:aws-translators');

  buildFilters = function(target, filters) {
    var filter, _i, _len, _results;
    if (filters) {
      _results = [];
      for (_i = 0, _len = filters.length; _i < _len; _i++) {
        filter = filters[_i];
        _results.push(scanFilterFunc(target, filter));
      }
      return _results;
    }
  };

  scanFilterFunc = function(target, filter) {
    target[filter.column] = {
      ComparisonOperator: filter.op || 'EQ',
      AttributeValueList: [{}]
    };
    target[filter.column].AttributeValueList[0][filter.type || 'S'] = filter.value;
    return target;
  };

  addAwsParams = function(target, params) {
    var key, value, _results;
    _results = [];
    for (key in params) {
      value = params[key];
      if (__indexOf.call(target, key) < 0 && key[0] === key[0].toUpperCase()) {
        _results.push(target[key] = value);
      } else {
        _results.push(void 0);
      }
    }
    return _results;
  };

  module.exports.processAllPages = function(deferred, dynamo, functionName, params) {
    var resultHandler, stats;
    stats = {
      Count: 0
    };
    resultHandler = function(err, result) {
      if (err) {
        return deferred.reject(err);
      }
      deferred.notify(dataTrans.fromDynamo(result.Items));
      stats.Count += result.Count;
      if (result.LastEvaluatedKey) {
        params.ExclusiveStartKey = result.LastEvaluatedKey;
        return dynamo[functionName](params, resultHandler);
      } else {
        return deferred.resolve(stats);
      }
    };
    dynamo[functionName](params, resultHandler);
    return deferred.promise;
  };

  module.exports.getKeySchema = function(tableDescription) {
    var getKeyAndType, hashKeyName, hashKeyType, rangeKeyName, rangeKeyType, _ref, _ref1;
    getKeyAndType = function(keyType) {
      var keyDataType, keyName, _ref, _ref1;
      keyName = (_ref = _.find(tableDescription.Table.KeySchema, function(key) {
        return key.KeyType === keyType;
      })) != null ? _ref.AttributeName : void 0;
      keyDataType = (_ref1 = _.find(tableDescription.Table.AttributeDefinitions, function(attribute) {
        return attribute.AttributeName === keyName;
      })) != null ? _ref1.AttributeType : void 0;
      return [keyName, keyDataType];
    };
    _ref = getKeyAndType('HASH'), hashKeyName = _ref[0], hashKeyType = _ref[1];
    _ref1 = getKeyAndType('RANGE'), rangeKeyName = _ref1[0], rangeKeyType = _ref1[1];
    return {
      hashKeyName: hashKeyName,
      hashKeyType: hashKeyType,
      rangeKeyName: rangeKeyName,
      rangeKeyType: rangeKeyType
    };
  };

  getKey = function(params, keySchema) {
    var key;
    if (!_.isObject(params)) {
      params = {
        hash: params + ''
      };
    }
    key = {};
    key[keySchema.hashKeyName] = {};
    key[keySchema.hashKeyName][keySchema.hashKeyType] = params.hash + '';
    if (params.range) {
      key[keySchema.rangeKeyName] = {};
      key[keySchema.rangeKeyName][keySchema.rangeKeyType] = params.range + '';
    }
    return key;
  };

  module.exports.deleteItem = function(params, options, callback, keySchema) {
    var awsParams;
    awsParams = {
      TableName: this.name,
      Key: getKey(params, keySchema)
    };
    addAwsParams(awsParams, options);
    return this.parent.dynamo.deleteItemPromise(awsParams);
  };

  module.exports.batchGetItem = function(params, callback, keySchema) {
    var awsParams, name;
    awsParams = {};
    awsParams.RequestItems = {};
    name = this.name;
    awsParams.RequestItems[this.name] = {
      Keys: _.map(params, function(param) {
        return getKey(param, keySchema);
      })
    };
    addAwsParams(awsParams, params);
    return this.parent.dynamo.batchGetItemPromise(awsParams).then(function(data) {
      return dataTrans.fromDynamo(data.Responses[name]);
    }).nodeify(callback);
  };

  module.exports.getItem = function(params, options, callback, keySchema) {
    var awsParams;
    awsParams = {
      TableName: this.name,
      Key: getKey(params, keySchema)
    };
    addAwsParams(awsParams, options);
    return this.parent.dynamo.getItemPromise(awsParams).then(function(data) {
      return dataTrans.fromDynamo(data.Item);
    }).nodeify(callback);
  };

  module.exports.queryByHashKey = function(key, callback, keySchema) {
    var awsParams, hashKeyName, hashKeyType;
    awsParams = {
      TableName: this.name,
      KeyConditions: {}
    };
    hashKeyName = keySchema.hashKeyName;
    hashKeyType = keySchema.hashKeyType;
    awsParams.KeyConditions[hashKeyName] = {
      ComparisonOperator: 'EQ',
      AttributeValueList: [{}]
    };
    awsParams.KeyConditions[hashKeyName].AttributeValueList[0][hashKeyType] = key;
    return this.parent.dynamo.queryPromise(awsParams).then(function(data) {
      return dataTrans.fromDynamo(data.Items);
    }).nodeify(callback);
  };

  module.exports.scan = function(params, options, callback, keySchema) {
    var awsParams, prop, val, _ref;
    if (params == null) {
      params = {};
    }
    awsParams = {
      TableName: this.name,
      ScanFilter: {},
      AttributesToGet: params.attrsGet || null,
      Limit: params.limit,
      TotalSegments: params.totalSegment,
      Segment: params.segment
    };
    if (params.ExclusiveStartKey != null) {
      awsParams.ExclusiveStartKey = {};
      _ref = params.ExclusiveStartKey;
      for (prop in _ref) {
        val = _ref[prop];
        awsParams.ExclusiveStartKey[prop] = dataTrans.toDynamo(val);
      }
    }
    buildFilters(awsParams.ScanFilter, params.filters);
    addAwsParams(awsParams, options);
    return this.parent.dynamo.scanPromise(awsParams).then(function(data) {
      return dataTrans.fromDynamo(data.Items);
    }).nodeify(callback);
  };

  module.exports.query = function(params, options, callback, keySchema) {
    var awsParams;
    if (params == null) {
      params = {};
    }
    awsParams = {
      TableName: this.name,
      IndexName: params.indexName,
      KeyConditions: {},
      QueryFilter: {}
    };
    buildFilters(awsParams.KeyConditions, params.keyConditions);
    buildFilters(awsParams.QueryFilter, params.filters);
    addAwsParams(awsParams, options);
    return this.parent.dynamo.queryPromise(awsParams).then(function(data) {
      return dataTrans.fromDynamo(data.Items);
    }).nodeify(callback);
  };

  module.exports.putItem = function(obj, options, callback) {
    var awsParams;
    awsParams = {
      TableName: this.name,
      Item: _.transform(obj, function(res, val, key) {
        return res[key] = dataTrans.toDynamo(val);
      })
    };
    addAwsParams(awsParams, options);
    return this.parent.dynamo.putItemPromise(awsParams);
  };

  module.exports.updateItem = function(params, obj, options, callback, keySchema) {
    var awsParams, expressionAttributeNames, expressionAttributeValues, i, key, updateExpression, _i, _len, _ref;
    key = getKey(params, keySchema);
    expressionAttributeValues = _.mapKeys(obj, function(value, key) {
      return ':' + key;
    });
    expressionAttributeValues = _.mapValues(expressionAttributeValues, function(value, key) {
      return dataTrans.toDynamo(value);
    });
    expressionAttributeNames = {};
    _ref = Object.keys(obj);
    for (i = _i = 0, _len = _ref.length; _i < _len; i = ++_i) {
      key = _ref[i];
      expressionAttributeNames["#" + key] = key;
    }
    updateExpression = 'SET ' + _.keys(_.mapKeys(obj, function(value, key) {
      return "#" + key + " = :" + key;
    })).join(',');
    awsParams = {
      TableName: this.name,
      Key: getKey(params, keySchema),
      ExpressionAttributeNames: expressionAttributeNames,
      ExpressionAttributeValues: expressionAttributeValues,
      UpdateExpression: updateExpression
    };
    addAwsParams(awsParams, options);
    return this.parent.dynamo.updateItemPromise(awsParams);
  };

}).call(this);
